part of coveralls.io;

/// Converts the specified [GitCommit] instance to a JSON object.
Map<String, dynamic> _gitCommitToJson(GitCommit commit) => commit.toJson();

/// Converts the specified list of [GitRemote] instances to a list of JSON objects.
List<Map<String, dynamic>> _gitRemotesToJson(List<GitRemote> remotes) => remotes.map((remote) => remote.toJson()).toList();

/// Represents a Git commit.
@JsonSerializable(includeIfNull: false)
class GitCommit {

  /// Creates a new commit.
  const GitCommit(this.id, {this.authorEmail, this.authorName, this.committerEmail, this.committerName, this.message});

  /// Creates a new commit from the specified [map] in JSON format.
  factory GitCommit.fromJson(Map<String, dynamic> map) => _$GitCommitFromJson(map);

  /// The author mail address.
  @JsonKey(name: 'author_email')
  final String authorEmail;

  /// The author name.
  @JsonKey(name: 'author_name')
  final String authorName;

  /// The committer mail address.
  @JsonKey(name: 'committer_email')
  final String committerEmail;

  /// The committer name.
  @JsonKey(name: 'committer_name')
  final String committerName;

  /// The commit identifier.
  final String id;

  /// The commit message.
  final String message;

  /// Converts this object to a [Map] in JSON format.
  Map<String, dynamic> toJson() => _$GitCommitToJson(this);

  /// Returns a [String] representation of this object.
  @override
  String toString() => 'GitCommit ${json.encode(this)}';
}

/// Represents Git data that can be used to display more information to users.
@JsonSerializable()
class GitData {

  /// Creates a new data.
  GitData(this.commit, {this.branch = '', Iterable<GitRemote> remotes}): remotes = List<GitRemote>.from(remotes ?? const <GitRemote>[]);

  /// Creates a new data from the specified [map] in JSON format.
  factory GitData.fromJson(Map<String, dynamic> map) => _$GitDataFromJson(map);

  /// The branch name.
  @JsonKey(defaultValue: '')
  String branch;

  /// The Git commit.
  @JsonKey(name: 'head', toJson: _gitCommitToJson)
  final GitCommit commit;

  /// The remote repositories.
  @JsonKey(toJson: _gitRemotesToJson)
  final List<GitRemote> remotes;

  /// Creates a new Git data from a repository located at the specified [path] (defaulting to the current working directory).
  /// This method relies on the availability of the Git executable in the system path.
  static Future<GitData> fromRepository([String path = '']) async {
    final commands = {
      'authorEmail': 'log -1 --pretty=format:%ae',
      'authorName': 'log -1 --pretty=format:%aN',
      'branch': 'rev-parse --abbrev-ref HEAD',
      'committerEmail': 'log -1 --pretty=format:%ce',
      'committerName': 'log -1 --pretty=format:%cN',
      'id': 'log -1 --pretty=format:%H',
      'message': 'log -1 --pretty=format:%s',
      'remotes': 'remote -v'
    };

    final workingDir = path.isNotEmpty ? path : Directory.current.path;
    for (final key in commands.keys) {
      final result = await Process.run('git', commands[key].split(' '), workingDirectory: workingDir);
      commands[key] = result.stdout.trim();
    }

    final remotes = <String, GitRemote>{};
    for (final remote in commands['remotes'].split(RegExp(r'\r?\n'))) {
      final parts = remote.replaceAll(RegExp(r'\s+'), ' ').split(' ');
      remotes.putIfAbsent(parts.first, () => GitRemote(parts.first, parts.length > 1 ? Uri.parse(parts[1]) : null));
    }

    return GitData(GitCommit.fromJson(commands), branch: commands['branch'], remotes: remotes.values);
  }

  /// Converts this object to a [Map] in JSON format.
  Map<String, dynamic> toJson() => _$GitDataToJson(this);

  /// Returns a [String] representation of this object.
  @override
  String toString() => 'GitData ${json.encode(this)}';
}

/// Represents a Git remote repository.
@JsonSerializable()
class GitRemote {

  /// Creates a new remote repository.
  GitRemote(this.name, [this.url]);

  /// Creates a new source file from the specified [map] in JSON format.
  factory GitRemote.fromJson(Map<String, dynamic> map) => _$GitRemoteFromJson(map);

  /// The remote's name.
  @JsonKey(defaultValue: '')
  final String name;

  /// The remote's URL.
  final Uri url;

  /// Converts this object to a [Map] in JSON format.
  Map<String, dynamic> toJson() => _$GitRemoteToJson(this);

  /// Returns a [String] representation of this object.
  @override
  String toString() => 'GitRemote ${json.encode(this)}';
}
